@@ -1,39 +1,60 @@
-//////////////////////////////////////////////////////////////////////
-//      $Id: kpackage.cpp,v 1.70.2.1 2001/08/18 11:01:17 binner Exp $ 
-// File  : kpackage.cpp
+/*
+** Copyright (C) 1999,2000 Toivo Pedaste <toivo@ucs.uwa.edu.au>
+**
 // Author: Damyan Pepper
 //         Toivo Pedaste
 //
 // See kpackage.h for more information.
-//////////////////////////////////////////////////////////////////////
+*/
+
+/*
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program in a file called COPYING; if not, write to
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
+*/
+
+/*
+** Bug reports and questions can be sent to kde-devel@kde.org
+*/
+
 
 #include "../config.h"
-extern "C"
-{
-#include <time.h>
-}  
-#include <unistd.h>
 
 #include <qdir.h>
-#include <qdragobject.h>
+#include <qlabel.h>
+#include <qframe.h>
 
+#include <kdebug.h>
+#include <kapplication.h>
+#include <kfiledialog.h>
+#include <kprogress.h>
 #include <kurl.h>
-#include <kapp.h>
+#include <kapplication.h>
 #include <kaccel.h>
-#include <kkeydialog.h>
+#include <kaction.h>
 #include <klocale.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <kstdaction.h>
 #include <kedittoolbar.h>
 #include <kmimemagic.h>
+#include <kurldrag.h>
 
 #include "kpackage.h"
-#include "pkgInterface.h"
 #include "managementWidget.h"
 #include "pkgOptions.h"
-#include "pkgInterface.h"
 #include "kio.h"
 #include "findf.h"
 #include "search.h"
@@ -43,10 +64,10 @@ extern "C"
 extern Opts *opts;
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
-KPKG::KPKG(KConfig *_config) 
+KPKG::KPKG(KConfig *_config)
   : KMainWindow(0)
 {
-  kpackage = new KPACKAGE(_config,this);
+  kpackage = new KPACKAGE(_config, this);
   setCentralWidget(kpackage);
 
   config =  kapp->config();
@@ -55,7 +76,7 @@ KPKG::KPKG(KConfig *_config)
   kpackage->management->readPSeparator();
 
   // Get a nice default size
-  resize(600,400);
+  resize(760,540);
 
   setupMenu();
   disableNext();
@@ -64,33 +85,7 @@ KPKG::KPKG(KConfig *_config)
   optiondialog = new Options(this);
 
   prop_restart = false;
-
-
-  int width, height;
-  width = config->readNumEntry("Width", 760);
-  height = config->readNumEntry("Height", 540);
-
-  if( width < minimumSize().width() ) {
-    width = minimumSize().width();
-  }
-  if( height < minimumSize().height() ) {
-    height = minimumSize().height();
-  }
-
-  resize(width, height);
-
-}
-
-// Destructor
-KPKG::~KPKG()
-{
-  //   object deletion - slows down exit
-  //    delete filemenu;
-  //    delete helpmenu;
-  //    delete kp; 
-  //    delete options; 
-  //    delete optiondialog; 
-  //    delete caches;
+  setAutoSaveSettings();
 }
 
 // Set up the menu
@@ -98,118 +93,104 @@ KPKG::~KPKG()
 void KPKG::setupMenu()
 {
 
-
-  keys = new KAccel(this);
-
   pack_open = KStdAction::open(kpackage, SLOT(fileOpen()),
-			  actionCollection());
-  pack_open->plugAccel(keys);
+			       actionCollection());
 
   recent = KStdAction::openRecent(this, SLOT(openRecent(const KURL&)),
 				  actionCollection());
   recent->loadEntries( config );
 
- pack_find = new KAction( i18n("Find &Package"), "find", 
-		       KStdAccel::key(KStdAccel::Find), kpackage,
-		       SLOT(find()), actionCollection(), "pack_find");
-  pack_find->plugAccel(keys);
+  pack_find = new KAction( i18n("Find &Package..."), "find",
+			   KStdAccel::shortcut(KStdAccel::Find), kpackage,
+			   SLOT(find()), actionCollection(), "pack_find");
 
-  pack_findf = new KAction( i18n("Find &File"), "filefind", 
-		       0, kpackage,
-		       SLOT(findf()), actionCollection(), "pack_findf");
-  pack_findf->plugAccel(keys);
+  pack_findf = new KAction( i18n("Find &File..."), "filefind",
+			    0, kpackage,
+			    SLOT(findf()), actionCollection(), "pack_findf");
 
-  kpack_reload = new KAction( i18n("&Reload"), "reload", 
-		       KStdAccel::key(KStdAccel::Reload), kpackage,
-		       SLOT(reload()), actionCollection(), "kpack_reload");
-  kpack_reload->plugAccel(keys);
+  kpack_reload = new KAction( i18n("&Reload"), "reload",
+			      KStdAccel::shortcut(KStdAccel::Reload), kpackage,
+			      SLOT(reload()), actionCollection(), "kpack_reload");
 
   (void) KStdAction::quit(kpackage, SLOT(fileQuit()),
-			  actionCollection())->plugAccel(keys);
+			  actionCollection());
 
-  pack_prev = KStdAction::back(kpackage->management, SLOT(previous()),
-			  actionCollection(),"pack_prev");
-  pack_prev->plugAccel(keys);
+  pack_prev = KStdAction::back(kpackage->management->treeList, SLOT(previous()),
+			       actionCollection(),"pack_prev");
 
-  pack_next = KStdAction::forward(kpackage->management, SLOT(next()),
-			     actionCollection(),"pack_next");
-  pack_next->plugAccel(keys);
+  pack_next = KStdAction::forward(kpackage->management->treeList, SLOT(next()),
+				  actionCollection(),"pack_next");
 
-  (void) (new KAction( i18n("&Expand Tree"), "ftout", 
+  (void) (new KAction( i18n("&Expand Tree"), "ftout",
 		       0, kpackage,
-		       SLOT(expandTree()), actionCollection(), "kpack_expand"))->plugAccel(keys);
+		       SLOT(expandTree()), actionCollection(), "kpack_expand"));
 
-  (void) (new KAction( i18n("&Collapse Tree"), "ftin", 
+  (void) (new KAction( i18n("&Collapse Tree"), "ftin",
 		       0, kpackage,
-		       SLOT(collapseTree()), actionCollection(), "kpack_collapse"))->plugAccel(keys);
+		       SLOT(collapseTree()), actionCollection(), "kpack_collapse"));
 
-  (void) (new KAction( i18n("Clear &Marked"), 
+  (void) (new KAction( i18n("Clear &Marked"), QString::null,
 		       0, kpackage,
-		       SLOT(clearMarked()), actionCollection(), "kpack_clear"))->plugAccel(keys);
+		       SLOT(clearMarked()), actionCollection(), "kpack_clear"));
+
+  (void) (new KAction( i18n("Mark &All"), QString::null,
+		       0, kpackage,
+		       SLOT(markAll()), actionCollection(), "kpack_markall"));
+
+  pack_install = new KAction( i18n("&Install"), QString::null,
+			      0, kpackage->management,
+			      SLOT(installSingleClicked()), actionCollection(), "install_single");
 
-  pack_install = new KAction( i18n("&Install"), 
-		       0, kpackage->management,
-		       SLOT(installSingleClicked()), actionCollection(), "install_single");
-  pack_install->plugAccel(keys);
   pack_install->setEnabled(false);
   kpackage->management->setInstallAction(pack_install);
 
 
-  pack_uninstall = new KAction( i18n("&Uninstall"),
-		       0, kpackage->management,
-		       SLOT(uninstallSingleClicked()), actionCollection(), "uninstall_single");
-  pack_uninstall->plugAccel(keys);
+  pack_uninstall = new KAction( i18n("&Uninstall"), QString::null,
+				0, kpackage->management,
+				SLOT(uninstallSingleClicked()), actionCollection(), "uninstall_single");
+
   pack_uninstall->setEnabled(false);
   kpackage->management->setUninstallAction(pack_uninstall);
 
 
-  (void) (new KAction( i18n("&Install Marked"),
+  (void) (new KAction( i18n("&Install Marked"), QString::null,
 		       0, kpackage->management,
-		       SLOT(installMultClicked()), actionCollection(), "install_marked"))->plugAccel(keys);
+		       SLOT(installMultClicked()), actionCollection(), "install_marked"));
 
-  (void) (new KAction( i18n("&Uninstall Marked"),
+  (void) (new KAction( i18n("&Uninstall Marked"), QString::null,
 		       0, kpackage->management,
-		       SLOT(uninstallMultClicked()), actionCollection(), "uninstall_marked"))->plugAccel(keys);
+		       SLOT(uninstallMultClicked()), actionCollection(), "uninstall_marked"));
 
-  pack_toolbar = KStdAction::showToolbar( this, SLOT(toggleToolBar()), actionCollection());
+  setStandardToolBarMenuEnabled(true);
 
   KStdAction::configureToolbars( this, SLOT(configureToolBars()),
-				 actionCollection())->plugAccel(keys);
+				 actionCollection());
 
-  KStdAction::saveOptions( this, SLOT(saveSettings()), actionCollection())->plugAccel(keys);
+  KStdAction::saveOptions( this, SLOT(saveSettings()), actionCollection());
 
-  KStdAction::keyBindings( this, SLOT(setKeys()), actionCollection())->plugAccel(keys);
+  KStdAction::keyBindings( guiFactory(), SLOT(configureShortcuts()), actionCollection());
 
-  (void) (new KAction( i18n("Configure &Kpackage..."), "configure", 
+  (void) (new KAction( i18n("Configure &KPackage..."), "configure",
 		       0, this,
-		       SLOT(setOptions()), actionCollection(), "kpack_options"))->plugAccel(keys);
+		       SLOT(setOptions()), actionCollection(), "kpack_options"));
 
-  (void) (new KAction( i18n("Clear package &Directory cache"),
+  (void) (new KAction( i18n("Clear Package &Folder Cache"), QString::null,
 		       0, this,
-		       SLOT(clearDCache()), actionCollection(), "clear_dcache"))->plugAccel(keys);
+		       SLOT(clearDCache()), actionCollection(), "clear_dcache"));
 
-  (void) (new KAction( i18n("Clear &Package cache"),
+  (void) (new KAction( i18n("Clear &Package Cache"), QString::null,
 		       0, this,
-		       SLOT(clearPCache()), actionCollection(), "clear_pcache"))->plugAccel(keys);
+		       SLOT(clearPCache()), actionCollection(), "clear_pcache"));
 
   int i;
   for (i = 0; i < kpinterfaceN; i++) {
     if (kpinterface[i]) {
-      kpinterface[i]->makeMenu(keys, actionCollection());
+      kpinterface[i]->makeMenu(actionCollection());
     }
   }
 
-  int toolPos = config->readNumEntry("ToolBarPos",KToolBar::Left);
-  int iconText = config->readNumEntry("IconText");
-  int iconSize = config->readNumEntry("IconSize",0);
-
-  keys->readSettings();
   //  urlList.setAutoDelete(TRUE);
   createGUI();
-
-  toolBar()->setBarPos((KToolBar::BarPosition)toolPos);
-  toolBar()->setIconText((KToolBar::IconText)iconText);
-  toolBar()->setIconSize(iconSize);
 }
 
 void KPKG::disableMenu()
@@ -233,15 +214,15 @@ void KPKG::enableMenu()
 void KPKG::disableNext() {
   pack_next->setEnabled(false);
 }
-  
+
 void KPKG::enableNext() {
   pack_next->setEnabled(true);
 }
-  
+
 void KPKG::disablePrevious() {
   pack_prev->setEnabled(false);
 }
-  
+
 void KPKG::enablePrevious() {
   pack_prev->setEnabled(true);
 }
@@ -249,24 +230,21 @@ void KPKG::openRecent(const KURL& url){
   kpackage->openNetFile( url );
 }
 
-void KPKG::add_recent_file(QString newfile){
+void KPKG::add_recent_file(const QString &newfile){
 
   KURL url = KURL(newfile);
 
   recent->addURL( url );
 }
 
-void KPKG::toggleToolBar() {
-  if(pack_toolbar->isChecked())
-    toolBar("mainToolBar")->show();
-  else
-    toolBar("mainToolBar")->hide();
-}
-	
-void  KPKG::configureToolBars() {
+void KPKG::configureToolBars() {
   KEditToolbar dlg(actionCollection());
-  if (dlg.exec())
-    createGUI();
+  connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(slotNewToolbarConfig()));
+  dlg.exec();
+}
+
+void KPKG::slotNewToolbarConfig() {
+  createGUI();
 }
 
 void KPKG::writeSettings(){
@@ -277,24 +255,10 @@ void KPKG::writeSettings(){
 
   config->setGroup("Kpackage");
 
-  config->writeEntry("Width", width());
-  config->writeEntry("Height", height());
-  config->writeEntry( "ToolBar", hide_toolbar ? "off" : "on" );
-
-  // Bars created floating don't work
-  if (toolBar()->barPos() == KToolBar::Floating)
-    config->writeEntry("ToolBarPos", (int)KToolBar::Left );
-  else
-    config->writeEntry("ToolBarPos", (int) toolBar()->barPos() );
-  config->writeEntry("IconText", (int) toolBar()->iconText() );
-  config->writeEntry("IconSize", (int) toolBar()->iconSize() );
-
   recent->saveEntries( config );
 
-  kpackage->management->writeTreePos();
-  kpackage->management->writeTreeType();
-
-  keys->writeSettings();
+  kpackage->management->treeList->writeTreePos();
+  kpackage->management->treeList->writeTreeType();
 
   config->sync();
 }
@@ -303,10 +267,6 @@ void KPKG::setOptions(){
   optiondialog->restore();
 }
 
-void KPKG::setKeys(){
-  KKeyDialog::configureKeys( keys ); 
-}
-
 void KPKG::saveSettings(){
   writeSettings();
 }
@@ -321,16 +281,16 @@ void KPKG::clearDCache(){
 
 void KPKG::saveProperties(KConfig *config )
 {
-    config->writeEntry("Name", kpackage->save_url.url());
+    config->writePathEntry("Name", kpackage->save_url.url());
 }
 
 
 void KPKG::readProperties(KConfig *config)
 {
-    QString entry = config->readEntry("Name"); // no default
+    QString entry = config->readPathEntry("Name"); // no default
     if (entry.isNull())
 	return;
-    kpackage->openNetFile(entry);
+    kpackage->openNetFiles(entry);
     prop_restart = true;
 }
 
@@ -341,7 +301,7 @@ void KPKG::closeEvent ( QCloseEvent *) {
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-KPACKAGE::KPACKAGE(KConfig *_config, QWidget *parent) 
+KPACKAGE::KPACKAGE(KConfig *_config, QWidget *parent)
   : QWidget(parent)
 {
 
@@ -355,7 +315,7 @@ KPACKAGE::KPACKAGE(KConfig *_config, QWidget *parent)
 
   file_dialog = NULL;
   findialog = NULL;
-  srchdialog = NULL;  
+  srchdialog = NULL;
 
 }
 
@@ -376,7 +336,7 @@ void KPACKAGE::resizeEvent(QResizeEvent *re)
 
 // Set up the mode widgets
 void KPACKAGE::setupModeWidgets()
-{ 
+{
   management = new managementWidget(this);
 
   for (int i = 0; i < kpinterfaceN; i++) {
@@ -405,13 +365,12 @@ void KPACKAGE::setupStatusBar()
 {
   statusbar = new QFrame(this);
   statusbar->setFrameStyle(QFrame::Raised | QFrame::Panel);
-  processProgress = new KProgress(0,100,100,KProgress::Horizontal,statusbar);
-  processProgress->setBarStyle(KProgress::Solid);
+  processProgress = new KProgress(100,statusbar);
   processProgress->setTextEnabled(FALSE);
 
   status = new QLabel(i18n("Management Mode"), statusbar);
 }
-  
+
 // Arrange the widgets nicely
 void KPACKAGE::arrangeWidgets()
 {
@@ -423,10 +382,10 @@ void KPACKAGE::arrangeWidgets()
   status->move(2,2);
   processProgress->resize(statusbar->width() / 4 - 4, 16);
   processProgress->move((statusbar->width() / 4) * 3 + 3, 2);
-  
+
   management->resize(width(),height() - 20);
 
-  for (i = 0; i < kpinterfaceN; i++) 
+  for (i = 0; i < kpinterfaceN; i++)
     if (kpinterface[i]) {
       kpinterface[i]->installation->resize(width(),height() - 20);
     }
@@ -458,37 +417,39 @@ void KPACKAGE::reload()
 void KPACKAGE::fileOpen()		// file->quit selected from menu
 {
     KFileDialog *box;
-    //    pkgInterface *type;
 
-    box = getFileDialog(i18n("Select package"));
-    
-    box->show();
-    
-    if (!box->result())   /* cancelled */
-      return;
-    if(box->selectedURL().isEmpty()) {  /* no selection */
-      return;
+    box = getFileDialog(i18n("Select Package"));
+
+    if( box->exec())
+    {
+        if(!box->selectedURL().isEmpty())
+        {
+            openNetFile( box->selectedURL() );
+        }
     }
-    
-    openNetFile( box->selectedURL() );
 }
 
 void KPACKAGE::clearMarked()
 {
-  management->clearMarked(management->treeList->firstChild());
+  management->treeList->clearMarked(management->treeList->firstChild());
+}
+
+void KPACKAGE::markAll()
+{
+  management->treeList->markAll(management->treeList->firstChild());
 }
 
 void KPACKAGE::expandTree()
 {
-  management->expandTree(management->treeList);
+  management->treeList->expandTree(management->treeList);
 }
 
 void KPACKAGE::collapseTree()
 {
-  management->collapseTree(management->treeList);
+  management->treeList->collapseTree(management->treeList);
 }
 
-pkgInterface *KPACKAGE::pkType(QString fname)
+pkgInterface *KPACKAGE::pkType(const QString &fname)
 {
   // Get the package information for this package
   char buf[51];
@@ -498,7 +459,7 @@ pkgInterface *KPACKAGE::pkType(QString fname)
   if (file) {
     fgets(buf,sizeof(buf)-1,file);
     buf[50] = 0;
- 
+
     // check enabled package handlers
     for (i = 0; i < kpinterfaceN; i++) {
       if (kpinterface[i]) {
@@ -527,34 +488,97 @@ pkgInterface *KPACKAGE::pkType(QString fname)
 }
 
 /////////////////////////////////////////////////////////////////////////
-
-void KPACKAGE::openNetFile(const KURL & _url )
-{
-  QString s = fetchNetFile(_url);
-  kpkg->add_recent_file(_url.url());
-  if (!s.isEmpty()) {
-    pkgInterface *type = pkType(s);
-
-    //    KMimeMagic *magic = KMimeMagic::self();
-    //    KMimeMagicResult *r = magic->findFileType(s);
-    //    printf("r=%s\n",(r->mimeType()).data());
-
-    if (type) {
-      packageInfo *pk = type->getPackageInfo('u',s, 0);
-      if (pk) {
-	pk->pkgFileIns(s);
-	installPackage(s,type);	// start installing this package
-      }
+int KPACKAGE::typeIndex(pkgInterface *type) {
+  int i;
+  for (i = 0; i < kpinterfaceN; i++) {
+    if (type ==  kpinterface[i]) {
+      return i;
     }
   }
+  return -1;
 }
 
+void KPACKAGE::openNetFiles (const QStringList &urls, bool install )
+{
+  QStringList files;
+  int i;
+  int index;
+  QPtrList<packageInfo> **lst = new QPtrList<packageInfo>*[kpinterfaceN];
+  packageInfo *pk = 0;
+
+  kdDebug() << "openNetFiles\n";
+
+  for (QStringList::ConstIterator it = urls.begin(); it != urls.end(); ++it) {
+    files.append(fetchNetFile(*it));
+    kpkg->add_recent_file(*it);
+  }
+
+  for (i = 0; i < kpinterfaceN; i++) {
+    if (kpinterface[i]) {
+      lst[i] = new QPtrList<packageInfo>;
+    }
+  }
+  
+  for (QStringList::Iterator t = files.begin(); t != files.end(); ++t) {
+    pkgInterface *type = pkType(*t);
+    index = typeIndex(type);
+    if (index >= 0) {
+      pk = type->getPackageInfo('u', *t, 0);
+      if (pk) {
+	pk->pkgFileIns(*t);
+	lst[index]->insert(0,pk);
+        }
+      }
+    }
+
+  if (install)
+    for (i = 0; i < kpinterfaceN; i++) {
+      if (kpinterface[i]) {
+	if ( lst[i]->count() > 0) {
+	  kpinterface[i]->installation->setup(lst[i],kpinterface[i]->head);
+	  if (kpinterface[i]->installation->exec()) {
+	    for (packageInfo *inf = lst[i]->first(); inf != 0; inf = lst[i]->next()) {
+	      kpackage->management->updatePackage(inf,TRUE);
+	    }
+	  }
+	}
+      }
+    } else {
+      if (pk) {
+	KpTreeListItem *pt = pk->item;
+	// NOT the best place for this CODE
+	kpackage->management->tabChanged(Opts::ALL);
+	if (pt)
+	  kpackage->management->packageHighlighted(pt);
+      }
+    }
+
+  // Dealloc memory
+  for (i = 0; i < kpinterfaceN; i++) {
+    if (kpinterface[i]) {
+	delete lst[i];
+    }
+  }
+  delete [] lst;
+}
+
+void KPACKAGE::openNetFile(const KURL &url, bool install )
+{
+  openNetFiles(url.url(), install);
+}
+
+//    KMimeMagic *magic = KMimeMagic::self();
+//    KMimeMagicResult *r = magic->findFileType(s);
+    //    printf("r=%s\n",(r->mimeType()).data());
+
+
+
 QString KPACKAGE::getFileName(const KURL & url, QString &cacheName )
 {
   QString none  = "";
   QString fname = "";
 
-  if ( url.isMalformed() )  {
+  if ( !url.isValid() )  {
     KpMsgE(i18n("Malformed URL: %1").arg(url.url()),TRUE);
   } else {
 
@@ -563,22 +587,12 @@ QString KPACKAGE::getFileName(const KURL & url, QString &cacheName )
       cacheName = url.path();
       fname = url.path();
     } else {
-    
+
       QString tmpd = cacheObj::PDir();
       if (!tmpd.isEmpty()) {
 
 	QString cacheFile = tmpd + url.fileName();
-	
-	//	QString file = url.path();
-	//	int pt = file.findRev('/'); // could use KURL::fileName instead (David)
-	//	pt++;
-	//	if (pt >= 0) {  			// file name for cache
-	//	  cacheFile = tmpd + file.right(file.length() - pt);
-	//	} else {
-	//	  cacheFile = tmpd + file;
-	//	}
 
-	
 	cacheName = cacheFile;
 	QFileInfo f(cacheFile);
 	if (f.exists() && (opts->DCache != Opts::NEVER)) {
@@ -589,7 +603,24 @@ QString KPACKAGE::getFileName(const KURL & url, QString &cacheName )
   }
   return fname;
 }
-     
+
+bool KPACKAGE::isFileLocal( const KURL & url )
+{
+  QString cf;
+
+  QString f = getFileName(url, cf);
+
+  if (cf.isEmpty()) {
+    return false;
+  } else {
+    if (!f.isEmpty()) {
+      return true;
+    } else {
+      return false;
+    }
+  }
+}
+
 QString KPACKAGE::fetchNetFile( const KURL & url )
 {
 
@@ -606,21 +637,24 @@ QString KPACKAGE::fetchNetFile( const KURL & url )
     } else {
       save_url = url;
 
-      setStatus(i18n("Starting Kio"));
-  
+      setStatus(i18n("Starting KIO"));
+
       Kio kio;
 
       if (kio.download(url, cf)) {
-	setStatus(i18n("Kio finished"));
-	QFileInfo f(cf);
-	if (!(f.exists() && f.size() > 0)) {
-	  unlink(QFile::encodeName(cf));
+	setStatus(i18n("KIO finished"));
+	QFileInfo fi(cf);
+	if (!(fi.exists() && fi.size() > 0)) {
+          unlink(QFile::encodeName(cf));
 	  return "";
 	} else {
+          CacheList cl(fi.dirPath());
+          cl.append(fi.fileName());
+          cl.write();
 	  return cf;
 	}
       } else {
-	setStatus(i18n("Kio failed"));
+	setStatus(i18n("KIO failed"));
 	return "";
       }
     }
@@ -630,12 +664,15 @@ QString KPACKAGE::fetchNetFile( const KURL & url )
 /////////////////////////////////////////////////////////////////////////
 void KPACKAGE::fileOpenUrl(){
 
-  KLineEditDlg geturl( i18n("Open Location:"), save_url.prettyURL(), this );
+  bool ok;
 
-  if ( geturl.exec() )
+  QString url = KInputDialog::getText( QString::null,
+      i18n( "Open location:" ), save_url.prettyURL(), &ok, this );
+
+  if ( ok )
     {
-      kpkg->add_recent_file(geturl.text());
-      openNetFile( geturl.text() );
+      kpkg->add_recent_file( url );
+      openNetFile( url );
     }
 }
 
@@ -653,7 +690,7 @@ void KPACKAGE::findf(){
     findialog = new FindF(this);
 }
 
-KFileDialog* KPACKAGE::getFileDialog(QString captiontext)
+KFileDialog* KPACKAGE::getFileDialog(const QString &captiontext)
 {
 
   if(!file_dialog) {
@@ -677,44 +714,19 @@ KFileDialog* KPACKAGE::getFileDialog(QString captiontext)
 
 void KPACKAGE::dragEnterEvent(QDragEnterEvent* e)
 {
-  e->accept(QUriDrag::canDecode(e));
+  e->accept(KURLDrag::canDecode(e));
 }
 
 void KPACKAGE::dropEvent(QDropEvent *de) // something has been dropped
 {
-  int i = 0;
+  KURL::List list;
+  if (!KURLDrag::decode(de, list) || list.isEmpty())
+     return;
 
-  // Call openNetFile with the first URL, save the rest to be processed
-  QStrList list;
-  QUriDrag::decode(de, list);
-  kpkg->urlList.clear();
-
-  for ( const char *s = list.first();
-	s != 0;
-	s = list.next(), i++ ) {
-    if (i == 0)
-      openNetFile(s);
-    else
-      kpkg->urlList.append(s);
-  }
+  openNetFiles(list.toStringList());
 }
 
-void KPACKAGE::installPackage(QString filename, pkgInterface *type)	// start installing package
-{
-  // type->installation->installPackage(name); // so tell the installation widget about it
-  packageInfo *pk = type->getPackageInfo('u',filename, 0);
-  if (pk) {
-    pk->setFilename(filename);
-    management->doChangePackage(pk);
-    type->installation->setup(pk, type->head); 
-    int result = type->installation->exec();
-    if (result) {
-      management->updatePackage(pk,TRUE);
-    }
-  }
-}
-
-void KPACKAGE::setStatus(QString s)	// set the text in the status bar
+void KPACKAGE::setStatus(const QString &s)	// set the text in the status bar
 {
   status->setText(s);
   kapp->processEvents();	// refresh the screen
@@ -724,7 +736,7 @@ QString KPACKAGE::getStatus()	// get the text in the status bar
 {
   if(status)
     return status->text();
-  else 
+  else
     return "";
 }
 
