@@ -1,5 +1,4 @@
-/* pkgOptions.cpp
-**
+/*
 ** Copyright (C) 1999,2000 Toivo Pedaste <toivo@ucs.uwa.edu.au>
 **
 */
@@ -17,8 +16,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -148,13 +147,13 @@ void pkgOptions::setupWidgets(QPtrList<param> &pars)
   resize(800, 400);
 }
 
-void pkgOptions::setup(packageInfo *p, QString type) {
+void pkgOptions::setup(packageInfo *p, const QString &type) {
   QPtrList<packageInfo> *pl = new QPtrList<packageInfo>;
   pl->append(p);
   setup(pl,type);
 }
 
-void pkgOptions::setup(QPtrList<packageInfo> *pl, QString)
+bool pkgOptions::setup(QPtrList<packageInfo> *pl, const QString &)
 {
  QString s;
  modified = FALSE;
@@ -177,8 +176,13 @@ void pkgOptions::setup(QPtrList<packageInfo> *pl, QString)
   }
 
   packageInfo *pk;
+  bool cancel;
   if (clist.count() > 0) {
-    rlist = pkgInt->listInstalls(clist, installer);
+    rlist = pkgInt->listInstalls(clist, installer, cancel);
+    if (cancel) {
+      reject();
+      return false;
+    }
     for ( QStringList::Iterator it = rlist.begin(); it != rlist.end(); ++it ) {
       if (!dict[*it]) {
 	plist.append(*it);
@@ -198,11 +202,7 @@ void pkgOptions::setup(QPtrList<packageInfo> *pl, QString)
     }
   }
 
- if (plist.count() == 1) {
-   s = i18n("%1: 1 %2 Package").arg(insType).arg(pkgInt->name);
- } else {
-   s = i18n("%1: %2 %3 Packages").arg(insType).arg(plist.count()).arg(pkgInt->name);
- }
+ s = i18n("%1: 1 %2 Package","%1: %n %2 Packages",plist.count()).arg(insType,pkgInt->name);
  title->setText(s);
 
   for (QStringList::Iterator pit = plist.begin(); pit != plist.end(); ++pit ) {
@@ -210,6 +210,7 @@ void pkgOptions::setup(QPtrList<packageInfo> *pl, QString)
     new QListViewItem(packages, *pit);
   }
   cancelButton->setGuiItem(KStdGuiItem::cancel());
+  return TRUE;
 }
 
 // install button has been clicked....so install the package
@@ -300,14 +301,15 @@ void pkgOptions::cancelButtonClicked()
     accept();
 }
 
-void pkgOptions::hideEvent ( QHideEvent *h ) {
+void pkgOptions::closeEvent ( QCloseEvent * e ) {
+  kdDebug() << "pkgOptions::QCloseEvent\n";
   terminate();
-    
-  QWidget::hideEvent (h);
+   
+  QWidget::closeEvent (e);
 }
 
 void pkgOptions::showEvent ( QShowEvent *e ) {
-  //  kdDebug() << "SHOW\n";
+  //  kdDebug() << "pkgOptions::showEvent\n";
   getKeep();
 
   modified = FALSE;
