@@ -16,8 +16,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -75,7 +75,7 @@ public:
   void collectDepends(packageInfo *p, const QString &name, int src);
   bool parseName(const QString &name, QString *n, QString *v);
 
-  QString provMap(QString p);
+  QString provMap(const QString &p);
 
 public slots:
   void setLocation();
@@ -92,16 +92,16 @@ private:
 
   QString packageQuery();
   
-  QStringList getIFileList(packageInfo *p);
-  QStringList getUFileList(QString fn);
+  QStringList getIFileList( packageInfo *p );
+  QStringList getUFileList( const QString &fn );
 
-  QStringList getIChangeLog(packageInfo *p);
-  QStringList getUChangeLog(QString fn);
+  QStringList getIChangeLog( packageInfo *p );
+  QStringList getUChangeLog( const QString &fn );
 
-  packageInfo *getIPackageInfo( QString name);
-  packageInfo *getUPackageInfo( QString name);
+  packageInfo *getIPackageInfo( const QString &name);
+  packageInfo *getUPackageInfo( const QString &name);
 
-  QString quotePath( QString path);
+  QString quotePath( const QString &path);
 
  };
 
